/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <https://www.gnu.org/licenses/>.
 */
package com.gmail.filoghost.holographicdisplays.nms.v1_17_R1;

import com.gmail.filoghost.holographicdisplays.api.line.HologramLine;
import com.gmail.filoghost.holographicdisplays.nms.interfaces.entity.NMSEntityBase;
import com.gmail.filoghost.holographicdisplays.nms.interfaces.entity.NMSSlime;
import com.gmail.filoghost.holographicdisplays.util.ConsoleLogger;
import com.gmail.filoghost.holographicdisplays.util.reflection.ReflectField;
import com.google.common.collect.ImmutableList;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.chat.IChatBaseComponent;
import net.minecraft.server.level.EntityPlayer;
import net.minecraft.sounds.SoundEffect;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.damagesource.EntityDamageSource;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.EntityTypes;
import net.minecraft.world.entity.monster.EntitySlime;
import net.minecraft.world.level.World;
import net.minecraft.world.phys.AxisAlignedBB;
import org.bukkit.Bukkit;
import org.bukkit.craftbukkit.v1_17_R1.entity.CraftEntity;
import org.bukkit.event.player.PlayerInteractEntityEvent;

import java.util.logging.Level;

public class EntityNMSSlime extends EntitySlime implements NMSSlime {
	
	private static final ReflectField<Entity> VEHICLE_FIELD = new ReflectField<>(Entity.class, "au");

	private HologramLine parentPiece;
	private CraftEntity customBukkitEntity;
	
	public EntityNMSSlime(World world, HologramLine parentPiece) {
		super(EntityTypes.aD, world);
		super.setPersistent();
		super.collides = false;
		a(0.0F, 0.0F);
		setSize(1, false);
		setInvisible(true);
		this.parentPiece = parentPiece;
		forceSetBoundingBox(new NullBoundingBox());
	}
	
	@Override
	public void tick() {
		// Disable normal ticking for this entity.
		
		// So it won't get removed.
		super.R /* tickCount */ = 0;
	}
	
	@Override
	public void inactiveTick() {
		// Disable normal ticking for this entity.
		
		// So it won't get removed.
		super.R /* tickCount */ = 0;
	}
	
	@Override
	public void setPosition(double d0, double d1, double d2) {
		// Do not change it!
	}
	
	public void forceSetBoundingBox(AxisAlignedBB boundingBox) {
		super.a(boundingBox);
	}
	
	@Override
	public void saveData(NBTTagCompound nbttagcompound) {
		// Do not save NBT.
	}

	@Override
	public boolean d(NBTTagCompound nbttagcompound) {
		// Do not save NBT.
		return false;
	}

	@Override
	public boolean e(NBTTagCompound nbttagcompound) {
		// Do not save NBT.
		return false;
	}
	
	@Override
	public NBTTagCompound save(NBTTagCompound nbttagcompound) {
		// Do not save NBT.
		return nbttagcompound;
	}
	
	@Override
	public void load(NBTTagCompound nbttagcompound) {
		// Do not load NBT.
	}
	
	@Override
	public void loadData(NBTTagCompound nbttagcompound) {
		// Do not load NBT.
	}
	
	@Override
	public boolean damageEntity(DamageSource damageSource, float amount) {
		if (damageSource instanceof EntityDamageSource) {
			EntityDamageSource entityDamageSource = (EntityDamageSource) damageSource;
			if (entityDamageSource.getEntity() instanceof EntityPlayer) {
				Bukkit.getPluginManager().callEvent(new PlayerInteractEntityEvent(((EntityPlayer) entityDamageSource.getEntity()).getBukkitEntity(), getBukkitEntity())); // Bukkit takes care of the exceptions
			}
		}
		return false;
	}
	
	@Override
	public boolean isInvulnerable(DamageSource source) {
		/*
		 * The field Entity.invulnerable is private.
		 * It's only used while saving NBTTags, but since the entity would be killed
		 * on chunk unload, we prefer to override isInvulnerable().
		 */
	    return true;
	}
	
	@Override
	public boolean isCollidable() {
		return false;
	}

	@Override
	public void setCustomName(IChatBaseComponent ichatbasecomponent) {
		// Locks the custom name.
	}
	
	@Override
	public void setCustomNameVisible(boolean visible) {
		// Locks the custom name.
	}
	
	@Override
	public void playSound(SoundEffect soundeffect, float f, float f1) {
	    // Remove sounds.
	}
	
	@Override
	public void killEntity() {
		// Prevent being killed.
	}
	
	@Override
	public CraftEntity getBukkitEntity() {
		if (customBukkitEntity == null) {
			customBukkitEntity = new CraftNMSSlime(super.getWorld().getCraftServer(), this);
	    }
		return customBukkitEntity;
	}

	@Override
	public boolean isDeadNMS() {
		return super.isRemoved();
	}
	
	@Override
	public void killEntityNMS() {
		super.setRemoved(RemovalReason.b /* DISCARDED */);
	}
	
	@Override
	public void setLocationNMS(double x, double y, double z) {
		super.setPosition(x, y, z);
	}
	
	@Override
	public int getIdNMS() {
		return super.getId();
	}
	
	@Override
	public HologramLine getHologramLine() {
		return parentPiece;
	}

	@Override
	public org.bukkit.entity.Entity getBukkitEntityNMS() {
		return getBukkitEntity();
	}
	
	@Override
	public void setPassengerOfNMS(NMSEntityBase vehicleBase) {
		if (vehicleBase == null || !(vehicleBase instanceof Entity)) {
			// It should never dismount
			return;
		}
		
		Entity entity = (Entity) vehicleBase;
		
		try {
			if (super.getVehicle() != null) {
				Entity oldVehicle = super.getVehicle();
				VEHICLE_FIELD.set(this, null);
				oldVehicle.at /* passengers */ = ImmutableList.of();
			}

			VEHICLE_FIELD.set(this, entity);
			entity.at /* passengers */ = ImmutableList.of(this);

		} catch (Throwable t) {
			ConsoleLogger.logDebug(Level.SEVERE, "Couldn't set passenger", t);
		}
	}
}
